import os
from typing import List
import numpy as np

# Funktionen
def list_to_float(data: List[str]) -> List[float]:
    """
    Converts a list of strings into a list of floats
    """
    float_list = []
    for i in data:
        float_list.append(float(i))
    return float_list

def mean(data: List[float]) -> float:
    """
    Calculates the mean value from a list of floats
    """
    return sum(data) / len(data)

def moving_average(data: List[float], window_size: int = 5) -> List[float]:
    """
    Calculates the moving average of a list of floats with a given window size
    Default window size is 5
    """
    mov_avg = []

    for current_index in range(1, len(data)+1):
        end = current_index
        start = current_index - window_size

        if start < 0:
            start = 0

        #print(f"Sublist from index {start} to index {end-1} is averaged")

        sub_list = data[start:end]
        mov_avg.append(sum(sub_list) / len(sub_list))

    return mov_avg

def mse(data1: List[float], data2: List[float]) -> float:
    """
    Calculates the mean squared error between two lists of floats
    """
    n = len(data1)
    mse_sum = 0.0

    for i in range(n):
        mse_sum = mse_sum + (data1[i] - data2[i])**2

    return mse_sum / n

# Daten einlesen
path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'keyboard_clean.csv')
raw_data = np.loadtxt(path, delimiter=",", skiprows=1, dtype=str)

time_str = raw_data[:, 0].tolist()
acc_x_str = raw_data[:, 1].tolist()
acc_y_str = raw_data[:, 2].tolist()
acc_z_str = raw_data[:, 3].tolist()

# Daten umwandeln
time = list_to_float(time_str)
acc_x = list_to_float(acc_x_str)
acc_y = list_to_float(acc_y_str)
acc_z = list_to_float(acc_z_str)

# Mittelwert berechnen
print(f"acc_x_mean = {mean(acc_x):.2f} ms^-2")
print(f"acc_y_mean = {mean(acc_y):.2f} ms^-2")
print(f"acc_z_mean = {mean(acc_z):.2f} ms^-2")

# Glättung
acc_x_mov5 = moving_average(acc_x)
acc_y_mov5 = moving_average(acc_y)
acc_z_mov5 = moving_average(acc_z)

# Mean Squared Error
print(f"MSE_x = {mse(acc_x, acc_x_mov5)*100:.2f}%")
print(f"MSE_y = {mse(acc_y, acc_y_mov5)*100:.2f}%")
print(f"MSE_z = {mse(acc_z, acc_z_mov5)*100:.2f}%")
